/*************************************************
* Filter Header File                             *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_FILTER_H__
#define OPENCL_FILTER_H__

#include <opencl/opencl.h>

namespace OpenCL {

/*************************************************
* Filter Base Class                              *
*************************************************/
class Filter
   {
   public:
      struct Invalid_Port_Number : public Invalid_Argument
         { Invalid_Port_Number() :
              Invalid_Argument("Filter: Invalid port number was used") {} };
      virtual ~Filter() {}
   protected:
      virtual void send(const byte[], u32bit);
      void send(byte input) { send(&input, 1); }
      void attach(Filter*);
      void set_port(u32bit) throw(Invalid_Port_Number);
      u32bit total_ports()  const { return port_count; }
      u32bit current_port() const { return port_num; }
      Filter(u32bit = 1);
   private:
      friend class Pipe;
      friend class Fork;
      friend void destroy(Filter*&);
      Filter(const Filter&) : port_count(0) {}
      void operator=(const Filter&) {}
      virtual void write(const byte[], u32bit) = 0;
      virtual void final() {}
      virtual void close();
      void add(const class SecureQueue&, Filter*);
      Filter* get_next() const { if(next) return next[port_num]; return 0; }
      const u32bit port_count;
      SecureVector<byte> write_queue;
      Filter** next;
      u32bit port_num;
   };

/*************************************************
* Destroy this Filter and it's children          *
*************************************************/
void destroy(Filter*&);

}

#endif
