/*************************************************
* Pipe Header File                               *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_PIPE_H__
#define OPENCL_PIPE_H__

#include <iosfwd>
#include <cstdio>
#include <opencl/filter.h>

namespace OpenCL {

/*************************************************
* Pipe                                           *
*************************************************/
class Pipe
   {
   public:
      void write(const byte input[], u32bit len) { pipe->write(input, len); }
      void write(byte input) { write(&input, 1); }
      void write(const std::string&);
      void finish(const byte in[], u32bit len) { write(in, len); final(); }

      u32bit read(byte[], u32bit);
      u32bit read(byte& output) { return read(&output, 1); }
      SecureVector<byte> read_all();
      std::string read_all_as_string();

      u32bit remaining() const;

      void final() { pipe->close(); }

      void attach(Filter* filter) { pipe->attach(filter); }
      void reset() { destroy(pipe); init(); }

      Pipe(Filter* f1 = 0, Filter* f2 = 0, Filter* f3 = 0, Filter* f4 = 0)
         { init(); attach(f1); attach(f2); attach(f3); attach(f4); }
      Pipe(Filter*[], u32bit);
      ~Pipe() { destroy(pipe); }
   private:
      struct NullFilter : public Filter
         { void write(const byte in[], u32bit len) { send(in, len); } };
      Pipe(const Pipe&) {}
      void operator=(const Pipe&) {}
      void init();
      class SecureQueue* end() const;
      Filter* pipe;
      u32bit msg_num;
   };

/*************************************************
* Input and Ouput Operators for Pipe             *
*************************************************/
std::ostream& operator<<(std::ostream&, Pipe&);
std::istream& operator>>(std::istream&, Pipe&);

std::FILE* operator<<(std::FILE*, Pipe&);
std::FILE* operator>>(std::FILE*, Pipe&);

}

#endif
