/*************************************************
* Secure Memory Allocator Header File            *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_SECURE_ALLOCATION_H__
#define OPENCL_SECURE_ALLOCATION_H__

#include <cstring>
#include <opencl/util.h>

namespace OpenCL {

/*************************************************
* Get an allocator                               *
*************************************************/
class SecureAllocator* get_allocator(const std::string& = "");

/*************************************************
* Set the default allocator type                 *
*************************************************/
void set_default_allocator(const std::string&);

/*************************************************
* Secure Allocator Base Class                    *
*************************************************/
class SecureAllocator
   {
   public:
      virtual void* allocate(u32bit) const = 0;
      virtual void deallocate(void*, u32bit) const = 0;
      virtual ~SecureAllocator() {}
   };

/*************************************************
* Default Allocator                              *
*************************************************/
class DefaultAllocator : public SecureAllocator
   {
   public:
      void* allocate(u32bit) const;
      void deallocate(void*, u32bit) const;
   };


}

#endif
