/*************************************************
* Block Cipher Mode Base Source File             *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#include <opencl/modebase.h>

namespace OpenCL {

/*************************************************
* Block Cipher Mode Constructor                  *
*************************************************/
BlockCipherMode::BlockCipherMode(BlockCipher* block_cipher, u32bit block_size,
                                 const std::string& name, const BlockCipherKey& key,
                                 const BlockCipherModeIV& iv) :
   BLOCKSIZE(block_size), IVSIZE(block_size), mode_name(name)
   {
   cipher = block_cipher;
   buffer.resize(BLOCKSIZE);
   state.resize(BLOCKSIZE);
   cipher->set_key(key, key.length());
   reset_iv(iv);
   position = 0;
   }

/*************************************************
* Reset the IV                                   *
*************************************************/
void BlockCipherMode::reset_iv(const BlockCipherModeIV& new_iv)
   {
   if(new_iv.length() != state.size())
      throw Invalid_IVLength(name(), new_iv.length());
   state.copy(new_iv, new_iv.length());
   buffer.clear();
   position = 0;
   }

}
