/*************************************************
* Pipe Source File                               *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#include <opencl/pipe.h>
#include <opencl/secqueue.h>

namespace OpenCL {

/*************************************************
* Pipe Constructor                               *
*************************************************/
Pipe::Pipe(Filter* filter_array[], u32bit count)
   {
   init();
   for(u32bit j = 0; j != count; j++)
      attach(filter_array[j]);
   }

/*************************************************
* Initialize the Pipe                            *
*************************************************/
void Pipe::init()
   {
   pipe = new NullFilter;
   pipe->attach(new SecureQueue);
   }

/*************************************************
* Find the end of the pipe                       *
*************************************************/
SecureQueue* Pipe::end() const
   {
   Filter* current = pipe;
   while(current->get_next())
      current = current->get_next();
   SecureQueue* queue = static_cast<SecureQueue*>(current);
   return queue;
   }

/*************************************************
* Write a string into a Pipe                     *
*************************************************/
void Pipe::write(const std::string& str)
   {
   write((const byte*)str.c_str(), str.size());
   }

/*************************************************
* Read some data from the pipe                   *
*************************************************/
u32bit Pipe::read(byte output[], u32bit length)
   {
   return end()->read(output, length);
   }

/*************************************************
* Return all data in the pipe                    *
*************************************************/
SecureVector<byte> Pipe::read_all()
   {
   SecureVector<byte> buffer(remaining());
   read(buffer, buffer.size());
   return buffer;
   }

/*************************************************
* Return all data in the pipe as a string        *
*************************************************/
std::string Pipe::read_all_as_string()
   {
   static const u32bit BUFFERSIZE = DEFAULT_BUFFERSIZE;
   SecureBuffer<byte, BUFFERSIZE> buffer;
   std::string str;
   while(remaining())
      {
      u32bit got = read(buffer, BUFFERSIZE);
      str.append((char*)buffer.ptr(), got);
      }
   return str;
   }

/*************************************************
* Find out how many bytes are ready to read      *
*************************************************/
u32bit Pipe::remaining() const
   {
   return end()->size();
   }

}
